@@ -8,25 +8,26 @@ from modeltranslation.utils import build_localized_fieldname
 
 
 class Command(NoArgsCommand):
-    help = ('Updates the default translation fields of all or the specified '
-            'translated application using the value of the original field.')
+    help = ('Updates empty values of default translation fields using'
+            ' values from original fields (in all translated models).')
 
-    def handle(self, **options):
+    def handle_noargs(self, **options):
         verbosity = int(options['verbosity'])
         if verbosity > 0:
             self.stdout.write("Using default language: %s\n" % DEFAULT_LANGUAGE)
-        for model, trans_opts in translator._registry.items():
-            if model._meta.abstract:
-                continue
+        models = translator.get_registered_models(abstract=False)
+        for model in models:
             if verbosity > 0:
                 self.stdout.write("Updating data of model '%s'\n" % model)
-            for fieldname in trans_opts.fields:
-                def_lang_fieldname = build_localized_fieldname(fieldname, DEFAULT_LANGUAGE)
+            opts = translator.get_options_for_model(model)
+            for field_name in opts.fields.iterkeys():
+                def_lang_fieldname = build_localized_fieldname(field_name, DEFAULT_LANGUAGE)
 
                 # We'll only update fields which do not have an existing value
                 q = Q(**{def_lang_fieldname: None})
-                field = model._meta.get_field(fieldname)
+                field = model._meta.get_field(field_name)
                 if field.empty_strings_allowed:
                     q |= Q(**{def_lang_fieldname: ""})
 
-                model.objects.filter(q).rewrite(False).update(**{def_lang_fieldname: F(fieldname)})
+                model.objects.filter(q).rewrite(False).update(
+                    **{def_lang_fieldname: F(field_name)})
