@@ -5,27 +5,23 @@ django-linguo by Zach Mathew
 
 https://github.com/zmathew/django-linguo
 """
+from __future__ import with_statement  # Python 2.5 compatibility
 from django.db import models
 from django.db.models.fields.related import RelatedField
 from django.db.models.sql.where import Constraint
 from django.utils.tree import Node
 
 from modeltranslation import settings
-from modeltranslation.utils import build_localized_fieldname, get_language
-
-
-_registry = {}
+from modeltranslation.utils import (build_localized_fieldname, get_language,
+                                    auto_populate)
 
 
 def get_translatable_fields_for_model(model):
-    from modeltranslation import translator
-    if model not in _registry:
-        try:
-            _registry[model] = dict(
-                translator.translator.get_options_for_model(model).localized_fieldnames)
-        except translator.NotRegistered:
-            _registry[model] = None
-    return _registry[model]
+    from modeltranslation.translator import NotRegistered, translator
+    try:
+        return translator.get_options_for_model(model).fields
+    except NotRegistered:
+        return None
 
 
 def rewrite_lookup_key(model, lookup_key):
@@ -64,8 +60,9 @@ def rewrite_order_lookup_key(model, lookup_key):
 
 
 def get_fields_to_translatable_models(model):
+    from modeltranslation.translator import translator
     results = []
-    for field_name in model._meta.get_all_field_names():
+    for field_name in translator.get_options_for_model(model).fields.keys():
         field_object, modelclass, direct, m2m = model._meta.get_field_by_name(field_name)
         if direct and isinstance(field_object, RelatedField):
             if get_translatable_fields_for_model(field_object.related.parent_model) is not None:
@@ -74,13 +71,13 @@ def get_fields_to_translatable_models(model):
 
 
 class MultilingualQuerySet(models.query.QuerySet):
-    _rewrite = True
-
     def __init__(self, *args, **kwargs):
         super(MultilingualQuerySet, self).__init__(*args, **kwargs)
         self._post_init()
 
     def _post_init(self):
+        self._rewrite = True
+        self._populate = None
         if self.model and (not self.query.order_by):
             if self.model._meta.ordering:
                 # If we have default ordering specified on the model, set it now so that
@@ -93,12 +90,20 @@ class MultilingualQuerySet(models.query.QuerySet):
     # This method was not present in django-linguo
     def _clone(self, *args, **kwargs):
         kwargs.setdefault('_rewrite', self._rewrite)
+        kwargs.setdefault('_populate', self._populate)
         return super(MultilingualQuerySet, self)._clone(*args, **kwargs)
 
     # This method was not present in django-linguo
     def rewrite(self, mode=True):
         return self._clone(_rewrite=mode)
 
+    # This method was not present in django-linguo
+    def populate(self, mode='all'):
+        """
+        Overrides the translation fields population mode for this query set.
+        """
+        return self._clone(_populate=mode)
+
     def _rewrite_applied_operations(self):
         """
         Rewrite fields in already applied filters/ordering.
@@ -178,20 +183,29 @@ class MultilingualQuerySet(models.query.QuerySet):
         return super(MultilingualQuerySet, self).update(**kwargs)
     update.alters_data = True
 
+    # This method was not present in django-linguo
+    @property
+    def _populate_mode(self):
+        # Populate can be set using a global setting or a manager method.
+        if self._populate is None:
+            return settings.AUTO_POPULATE
+        return self._populate
+
     # This method was not present in django-linguo
     def create(self, **kwargs):
-        populate = kwargs.pop('_populate', settings.AUTO_POPULATE)
-        if populate:
-            translatable_fields = get_translatable_fields_for_model(self.model)
-            if translatable_fields is not None:
-                for key, val in kwargs.items():
-                    if key in translatable_fields:
-                        # Try to add value in every language
-                        for new_key in translatable_fields[key]:
-                            kwargs.setdefault(new_key, val)
-        # If not use populate feature, then normal rewriting will occur at model's __init__
-        # That's why it is not performed here - no reason to rewrite twice.
-        return super(MultilingualQuerySet, self).create(**kwargs)
+        """
+        Allows to override population mode with a ``populate`` method.
+        """
+        with auto_populate(self._populate_mode):
+            return super(MultilingualQuerySet, self).create(**kwargs)
+
+    # This method was not present in django-linguo
+    def get_or_create(self, **kwargs):
+        """
+        Allows to override population mode with a ``populate`` method.
+        """
+        with auto_populate(self._populate_mode):
+            return super(MultilingualQuerySet, self).get_or_create(**kwargs)
 
 
 class MultilingualManager(models.Manager):
@@ -200,6 +214,9 @@ class MultilingualManager(models.Manager):
     def rewrite(self, *args, **kwargs):
         return self.get_query_set().rewrite(*args, **kwargs)
 
+    def populate(self, *args, **kwargs):
+        return self.get_query_set().populate(*args, **kwargs)
+
     def get_query_set(self):
         qs = super(MultilingualManager, self).get_query_set()
         if qs.__class__ == models.query.QuerySet:
