@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
-from django.conf import global_settings
+from contextlib import contextmanager
+
 from django.utils.encoding import force_unicode
 from django.utils.translation import get_language as _get_language
 from django.utils.functional import lazy
@@ -37,7 +38,7 @@ build_localized_verbose_name = lazy(_build_localized_verbose_name, unicode)
 
 
 def _join_css_class(bits, offset):
-    if '-'.join(bits[-offset:]) in [l[0] for l in global_settings.LANGUAGES]:
+    if '-'.join(bits[-offset:]) in settings.AVAILABLE_LANGUAGES + ['en-us']:
         return '%s-%s' % ('_'.join(bits[:len(bits) - offset]), '_'.join(bits[-offset:]))
     return ''
 
@@ -89,9 +90,60 @@ def resolution_order(lang, override=None):
     First is always the parameter language, later are fallback languages.
     Override parameter has priority over FALLBACK_LANGUAGES.
     """
+    if not settings.ENABLE_FALLBACKS:
+        return (lang,)
     if override is None:
         override = {}
     fallback_for_lang = override.get(lang, settings.FALLBACK_LANGUAGES.get(lang, ()))
     fallback_def = override.get('default', settings.FALLBACK_LANGUAGES['default'])
     order = (lang,) + fallback_for_lang + fallback_def
     return tuple(unique(order))
+
+
+@contextmanager
+def auto_populate(mode='all'):
+    """
+    Overrides translation fields population mode (population mode decides which
+    unprovided translations will be filled during model construction / loading).
+
+    Example:
+
+        with auto_populate('all'):
+            s = Slugged.objects.create(title='foo')
+        s.title_en == 'foo' // True
+        s.title_de == 'foo' // True
+
+    This method may be used to ensure consistency loading untranslated fixtures,
+    with non-default language active:
+
+        with auto_populate('required'):
+            call_command('loaddata', 'fixture.json')
+    """
+    current_population_mode = settings.AUTO_POPULATE
+    settings.AUTO_POPULATE = mode
+    try:
+        yield
+    finally:
+        settings.AUTO_POPULATE = current_population_mode
+
+
+@contextmanager
+def fallbacks(enable=True):
+    """
+    Temporarily switch all language fallbacks on or off.
+
+    Example:
+
+        with fallbacks(False):
+            lang_has_slug = bool(self.slug)
+
+    May be used to enable fallbacks just when they're needed saving on some
+    processing or check if there is a value for the current language (not
+    knowing the language)
+    """
+    current_enable_fallbacks = settings.ENABLE_FALLBACKS
+    settings.ENABLE_FALLBACKS = enable
+    try:
+        yield
+    finally:
+        settings.ENABLE_FALLBACKS = current_enable_fallbacks
